# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.

"""
IdeaScaly: IdeaScale API client
"""

__version__ = '0.1'
__author__ = 'Jorge Saldivar'
__license__ = 'MIT'

from ideascaly.api import API
from ideascaly.auth import AuthNonSSO, AuthNonSSOMem, AuthSSO, AuthResearch
from ideascaly.error import IdeaScalyError
from ideascaly.models import Author, Idea, Campaign, Comment, Vote, JSONModel, ModelFactory, Model
from ideascaly.parsers import Parser, RawParser, JSONParser, ModelParser



# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.

import six


class IdeaScalyError(Exception):
    """IdeaScaly exception"""

    def __init__(self, reason, response=None):
        self.reason = six.text_type(reason)
        self.response = response
        Exception.__init__(self, reason)

    def __str__(self):
        return self.reason


# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.

import json
import requests

from ideascaly.error import IdeaScalyError
from ideascaly.utils import convert_to_utf8_str


def bind_api(**config):

    class APIMethod(object):

        api = config['api']
        path = config['path']
        payload_type = config.get('payload_type', None)
        payload_list = config.get('payload_list', False)
        method = config.get('method', 'GET')
        allowed_param = config.get('allowed_param', [])
        pagination_param = config.get('pagination_param', [])
        post_param = config.get('post_param', [])
        session = requests.Session()

        def __init__(self, args, kwargs):
            self.parser = kwargs.pop('parser', self.api.parser)
            self.post_data = {}  # kwargs.pop('post_data', None)
            self.session.headers = self.api.auth_handler.token
            self.build_parameters(args, kwargs)
            self.build_path(args, kwargs)

        def build_path(self, args, kwargs):
            order_keys = ['date-down', 'date-up', 'votes-down', 'votes-up', 'comments-down', 'random-down',
                          'amount-pledged-down', 'vote.and.comments-down', 'status-down', 'status.change.date-down']

            for index, arg in enumerate(args):
                if arg is None:
                    continue
                try:
                    self.path = self.path.replace("{%s}" % self.allowed_param[index], convert_to_utf8_str(arg))
                except IndexError:
                    raise IdeaScalyError('Wrong number of parameters supplied!')

            for k, arg in kwargs.items():
                if arg is None:
                    continue
                if k not in self.allowed_param:
                    continue
                self.path = self.path.replace("{%s}" % k, convert_to_utf8_str(arg))

            if 'campaign_id' in kwargs.keys():
                self.path = '/campaigns/' + convert_to_utf8_str(kwargs['campaign_id']) + self.path

            # set status key
            if 'status_key' in kwargs.keys() and 'status_key' in self.pagination_param:
                self.path = self.path + '/' + convert_to_utf8_str(kwargs['status_key'])

            # set pagination
            if 'page_number' in kwargs.keys() and 'page_number' in self.pagination_param and \
               'page_size' in kwargs.keys() and 'page_size' in self.pagination_param:
                if isinstance(kwargs['page_number'],int) and isinstance(kwargs['page_size'],int):
                    self.path = self.path + '/' + convert_to_utf8_str(kwargs['page_number'])
                    self.path = self.path + '/' + convert_to_utf8_str(kwargs['page_size'])
                else:
                    raise IdeaScalyError('Error with pagination parameters, they both have to be numeric')

            # set result order
            if 'order_key' in kwargs.keys() and 'order_key' in self.pagination_param:
                if kwargs['order_key'] in order_keys:
                    self.path = self.path + '/' + convert_to_utf8_str(kwargs['order_key'])
                else:
                    raise IdeaScalyError('Error with order key parameter, it must be one of these: %s' % order_keys)

            # create new users without sending a verification email
            if 'silent' in kwargs.keys():
                if kwargs['silent']:
                    self.path += '/create/silent'

        def build_parameters(self, args, kwargs):
            self.post_data = {}

            if self.post_param:
                for index, arg in enumerate(args):
                    if arg is None:
                        continue
                    try:
                        self.post_data[self.post_param[index]] = convert_to_utf8_str(arg)
                    except IndexError:
                        raise IdeaScalyError('Too many parameters supplied!')

            for k, arg in kwargs.items():
                if arg is None:
                    continue
                if k not in self.post_param:
                    continue
                if k in self.post_data:
                    raise IdeaScalyError('Multiple values for parameter %s supplied!' % k)

                if isinstance(arg, type('')):
                    self.post_data[k] = convert_to_utf8_str(arg)
                else:
                    self.post_data[k] = arg

        def execute(self):
            # Build the URL of the end-point
            url = self.api.url + self.path
            if self.api.community_url.find("http") == -1:
                full_url = 'http://' + self.api.community_url + url
            else:
                full_url = self.api.community_url + url

            # Execute request
            try:
                self.session.headers['content-type'] = 'application/json'
                json_data = json.dumps(self.post_data)
                resp = self.session.request(self.method, full_url, data=json_data, timeout=self.api.timeout)
            except Exception as e:
                raise IdeaScalyError('Failed to send request: %s' % e)

            # If an error was returned, throw an exception
            if resp.status_code and not 200 <= resp.status_code < 300:
                try:
                    error_msg = self.parser.parse_error(resp.text)
                except Exception:
                    error_msg = "IdeaScale %s error response" % resp.status_code
                raise IdeaScalyError(error_msg, resp)

            # Parse the response payload
            result = self.parser.parse(self, resp.text)

            # Save pagination result
            if 'pager_total_count' in resp.headers.keys():
                self.api.last_call_pagination = resp.headers

            return result

    def _call(*args, **kwargs):
        method = APIMethod(args, kwargs)
        return method.execute()

    return _call


# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.


class Auth(object):
    _token = {}

    @property
    def token(self):
        return self._token


class AuthNonSSO(Auth):

    def __init__(self, token):
        Auth.__init__(self)
        self._token = {'api_token':token}


class AuthNonSSOMem(Auth):

    def __init__(self, api_token, member_token):
        Auth.__init__(self)
        self._token = {'api_token': api_token, 'member_token': member_token}


class AuthSSO(Auth):

    def __init__(self, api_token, sso_token):
        Auth.__init__(self)
        self._token = {'api_token': api_token, 'sso_token': sso_token}


class AuthResearch(Auth):

    def __init__(self, token):
        Auth.__init__(self)
        self._token = {'research_api_token':token}



# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.

from ideascaly.utils import parse_datetime
from ideascaly.error import IdeaScalyError


class Model(object):

    def __init__(self, api=None):
        self._api = api

    def __getstate__(self):
        # pickle
        pickle = dict(self.__dict__)
        try:
            del pickle['_api']  # do not pickle the API reference
        except KeyError:
            pass
        return pickle

    @classmethod
    def parse(cls, api, json):
        """Parse a JSON object into a model instance."""
        raise NotImplementedError

    @classmethod
    def parse_list(cls, api, json_list):
        """
            Parse a list of JSON objects into
            a result set of model instances.
        """
        results = []
        for json_obj in json_list:
            if json_obj:
                obj = cls.parse(api, json_obj)
                results.append(obj)

        return results

    def __repr__(self):
        state = ['%s=%s' % (k, repr(v)) for (k, v) in vars(self).items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(state))


class Author(Model):

    @classmethod
    def parse(cls, api, json):
        author = cls(api)
        setattr(author, '_json', json)

        for k, v in json.items():
            setattr(author, k, v)

        return author

    @classmethod
    def parse_list(cls, api, json_list):
        if isinstance(json_list, list):
            item_list = json_list
        else:
            item_list = json_list['authors']

        results = []
        for obj in item_list:
            results.append(cls.parse(api, obj))

        return results


class Campaign(Model):

    @classmethod
    def parse(cls, api, json):
        campaign = cls(api)
        setattr(campaign, '_json', json)

        for k, v in json.items():
            setattr(campaign, k, v)

        return campaign


class Idea(Model):

    @classmethod
    def parse(cls, api, json):
        idea = cls(api)
        setattr(idea, '_json', json)

        for k, v in json.items():
            if k == 'authorInfo':
                author_model = getattr(api.parser.model_factory, 'author') if api else Author
                author = author_model.parse(api, v)
                setattr(idea, k, author)
            elif k == 'creationDateTime':
                setattr(idea, k, parse_datetime(v))
            elif k == 'editedAt':
                setattr(idea, k, parse_datetime(v))
            elif k == 'statusChangeDate':
                setattr(idea, k, parse_datetime(v))
            elif k == 'tags':
                tags = []
                for tag in v:
                    tags.append(tag)
                setattr(idea, k, tags)
            else:
                setattr(idea, k, v)

        return idea


class Vote(Model):

    @classmethod
    def parse(cls, api, json):
        vote = cls(api)
        setattr(vote, '_json', json)

        for k, v in json.items():
            if k == 'creationDate':
                setattr(vote, k, parse_datetime(v))
            else:
                setattr(vote, k, v)

        return vote


class Comment(Model):

    @classmethod
    def parse(cls, api, json):
        comment = cls(api)
        setattr(comment, '_json', json)

        for k, v in json.items():
            if k == 'authorInfo':
                author_model = getattr(api.parser.model_factory, 'author') if api else Author
                author = author_model.parse(api, v)
                setattr(comment, k, author)
            elif k == 'creationDateTime':
                setattr(comment, k, parse_datetime(v))
            elif k == 'editedAt':
                setattr(comment, k, parse_datetime(v))
            elif k == 'statusChangeDate':
                setattr(comment, k, parse_datetime(v))
            elif k == 'tags':
                tags = []
                for tag in v:
                    tags.append(tag)
                setattr(comment, k, tags)
            else:
                setattr(comment, k, v)

        return comment


class JSONModel(Model):

    @classmethod
    def parse(cls, api, json):
        return json


class ModelFactory(object):
    """
    Used by parsers for creating instances
    of models.
    """

    campaign = Campaign
    author = Author
    idea = Idea
    vote = Vote
    comment = Comment
    json = JSONModel



# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.

import six
import dateutil.parser


def parse_datetime(str_date):
    try:
        date_is = dateutil.parser.parse(str_date)
        return date_is
    except:
        return None

def parse_html_value(html):
    return html[html.find('>')+1:html.rfind('<')]


def parse_a_href(atag):
    start = atag.find('"') + 1
    end = atag.find('"', start)
    return atag[start:end]


def convert_to_utf8_str(arg):
    # written by Michael Norton (http://docondev.blogspot.com/)
    if isinstance(arg, six.text_type):
        arg = arg.encode('utf-8')
    elif not isinstance(arg, bytes):
        arg = six.text_type(arg).encode('utf-8')
    return arg


def import_simplejson():
    try:
        import simplejson as json
    except ImportError:
        try:
            import json  # Python 2.6+
        except ImportError:
            raise ImportError("Can't load a json library")

    return json


# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.

from ideascaly.models import ModelFactory
from ideascaly.utils import import_simplejson
from ideascaly.error import IdeaScalyError


class Parser(object):

    def parse(self, method, payload):
        """
        Parse the response payload and return the result.
        Returns a tuple that contains the result data and the cursors
        (or None if not present).
        """
        raise NotImplementedError

    def parse_error(self, payload):
        """
        Parse the error message from payload.
        If unable to parse the message, throw an exception
        and default error message will be used.
        """
        raise NotImplementedError


class RawParser(Parser):

    def __init__(self):
        pass

    def parse(self, method, payload):
        return payload

    def parse_error(self, payload):
        return payload


class JSONParser(Parser):

    payload_format = 'json'

    def __init__(self):
        self.json_lib = import_simplejson()

    def parse(self, method, payload):
        try:
            json = self.json_lib.loads(payload)
        except Exception as e:
            raise IdeaScalyError('Failed to parse JSON payload: %s' % e)

        return json

    def parse_error(self, payload):
        error = self.json_lib.loads(payload)
        if 'error' in error.keys():
            return error['error']
        else:
            if 'message' in error.keys():
                return error['message']
            else:
                return error['errors']


class ModelParser(JSONParser):

    def __init__(self, model_factory=None):
        JSONParser.__init__(self)
        self.model_factory = model_factory or ModelFactory

    def parse(self, method, payload):
        try:
            if method.payload_type is None:
                return
            model = getattr(self.model_factory, method.payload_type)
        except AttributeError:
            raise IdeaScalyError('No model for this payload type: %s' % method.payload_type)

        json = JSONParser.parse(self, method, payload)

        if method.payload_list:
            result = model.parse_list(method.api, json)
        else:
            result = model.parse(method.api, json)

        return result



# IdeaScaly
# Copyright 2015 Jorge Saldivar
# See LICENSE for details.

from ideascaly.binder import bind_api
from ideascaly.parsers import ModelParser


class API():
    """IdeaScale API http://support.ideascale.com/customer/portal/articles/1001563-ideascale-rest-api"""

    _community_url = ''
    _last_call_pagination = {}

    def __init__(self, auth_handler, api_url='/a/rest/', ver='v1', timeout=60, parser=None):

        """ Api instance Constructor
        :param auth_handler: authentication handler
        :param api_url: API url, default:/a/rest/
        :param ver: API version, default:v1
        :param timeout: delay before to consider the request as timed out in seconds, default:60
        :param parser: ModelParser instance to parse the responses, default:None
        """
        self.auth_handler = auth_handler
        self.url = api_url + ver
        self.timeout = timeout
        self.parser = parser or ModelParser()

    @property
    def community_url(self):
        return self._community_url

    @community_url.setter
    def community_url(self, value):
        self._community_url = value

    @property
    def last_call_pagination(self):
        return self._last_call_pagination

    @last_call_pagination.setter
    def last_call_pagination(self, values):
        self._last_call_pagination['pager_total_count'] = values['pager_total_count']
        self._last_call_pagination['pager_page_size'] = values['pager_page_size']
        self._last_call_pagination['pager_current_page_number'] = values['pager_current_page_number']
        self._last_call_pagination['pager_first_index'] = values['pager_first_index']
        self._last_call_pagination['pager_last_index'] = values['pager_last_index']

    # ----
    # Functions related with campaign actions
    # ----

    @property
    def get_campaigns(self):

        return bind_api(
            api=self,
            path='/campaigns',
            payload_type='campaign',
            payload_list=True
        )

    def get_ideas_campaign(self, *args, **kwargs):
        """ :allowed_param:'campaignId'
            :pagination_param: 'page_number', 'page_size', 'order_key'
        """
        return bind_api(
            api=self,
            path='/campaigns/{campaignId}/ideas',
            payload_type='idea',
            allowed_param=['campaignId'],
            payload_list=True,
            pagination_param=['page_number', 'page_size', 'order_key']
        )(*args, **kwargs)

    @property
    def get_active_ideas(self):
        return bind_api(
            api=self,
            path='/campaigns/active/ideas',
            payload_type='idea',
            payload_list=True
        )

    @property
    def get_archived_ideas(self):
        return bind_api(
            api=self,
            path='/campaigns/archived/ideas',
            payload_type='idea',
            payload_list=True
        )

    # ----
    # Functions related with idea actions
    # ----

    def get_ideas_in_review(self, **kwargs):
        """ :pagination_param: 'page_number', 'page_size', 'order_key'
        """
        return bind_api(
            api=self,
            path='/ideas/inreview',
            payload_type='idea',
            payload_list=True,
            pagination_param=['page_number', 'page_size', 'order_key']
        )(**kwargs)

    def get_ideas_in_progress(self, **kwargs):
        """ :pagination_param: 'page_number', 'page_size', 'order_key'
        """
        return bind_api(
            api=self,
            path='/ideas/inprogress',
            payload_type='idea',
            payload_list=True,
            pagination_param=['page_number', 'page_size', 'order_key']
        )(**kwargs)

    def get_ideas_complete(self, **kwargs):
        """ :pagination_param: 'page_number', 'page_size', 'order_key'
        """
        return bind_api(
            api=self,
            path='/ideas/complete',
            payload_type='idea',
            payload_list=True,
            pagination_param=['page_number', 'page_size', 'order_key']
        )(**kwargs)

    def get_votes_idea(self, *args, **kwargs):
        """ :allowed_param:'ideaId'
        """
        return bind_api(
            api=self,
            path='/ideas/{ideaId}/votes',
            payload_type='vote',
            allowed_param=['ideaId'],
            payload_list=True
        )(*args, **kwargs)

    def get_comments_idea(self, *args, **kwargs):
        """ :allowed_param:'ideaId'
        """
        return bind_api(
            api=self,
            path='/ideas/{ideaId}/comments',
            payload_type='comment',
            allowed_param=['ideaId'],
            payload_list=True
        )(*args, **kwargs)

    def get_idea_details(self, *args, **kwargs):
        """ :allowed_param:'ideaId'
        """
        return bind_api(
            api=self,
            path='/ideas/{ideaId}',
            payload_type='idea',
            allowed_param=['ideaId']
        )(*args, **kwargs)

    def get_top_ideas(self, **kwargs):
        """ :pagination_param: 'page_number', 'page_size', 'order_key'
        """
        return bind_api(
            api=self,
            path='/ideas/top',
            payload_type='idea',
            payload_list=True,
            pagination_param=['page_number', 'page_size', 'order_key']
        )(**kwargs)

    def get_recent_ideas(self, **kwargs):
        """ :pagination_param: 'page_number', 'page_size', 'order_key'
        """
        return bind_api(
            api=self,
            path='/ideas/recent',
            payload_type='idea',
            payload_list=True,
            pagination_param=['page_number', 'page_size', 'order_key']
        )(**kwargs)

    def get_hot_ideas(self, **kwargs):
        """ :pagination_param: 'page_number', 'page_size', 'order_key'
        """
        return bind_api(
            api=self,
            path='/ideas/hot',
            payload_type='idea',
            payload_list=True,
            pagination_param=['page_number', 'page_size', 'order_key']
        )(**kwargs)

    def create_idea(self, *args, **kwargs):
        """ :allowed_param: 'title', 'text', 'campaignId', 'tags' (optional), 'customFields' (optional)
        """
        return bind_api(
            api=self,
            path='/idea',
            method='POST',
            payload_type='idea',
            post_param=["title", "text", "campaignId", "tags", "customFields"]
        )(*args, **kwargs)

    def attach_media_to_idea(self, *args, **kwargs):
        # TODO
        raise NotImplementedError

    def delete_idea(self, *args, **kwargs):
        """ :allowed_param: 'ideaId'
        """
        return bind_api(
            api=self,
            path='/idea/{ideaId}/delete',
            method='DELETE',
            payload_type='idea',
            allowed_param=['ideaId']
        )(*args, **kwargs)

    def vote_up_idea(self, *args, **kwargs):
        """ :allowed_param: 'ideaId', 'myVote' (optional)
        """
        return bind_api(
            api=self,
            path='/ideas/{ideaId}/vote/up',
            method='POST',
            payload_type='vote',
            allowed_param=['ideaId'],
            post_param=['myVote']
        )(*args, **kwargs)

    def vote_down_idea(self, *args, **kwargs):
        """ :allowed_param: 'ideaId', 'myVote' (optional)
        """
        return bind_api(
            api=self,
            path='/ideas/{ideaId}/vote/down',
            method='POST',
            payload_type='vote',
            allowed_param=['ideaId'],
            post_param=['myVote']
        )(*args, **kwargs)

    def comment_idea(self, *args, **kwargs):
        """ :allowed_param: 'ideaId', 'text'
        """
        return bind_api(
            api=self,
            path='/ideas/{ideaId}/comment',
            method='POST',
            payload_type='comment',
            allowed_param=['ideaId'],
            post_param=['text']
        )(*args, **kwargs)

    @property
    def get_votes_ideas(self):
        return bind_api(
            api=self,
            path='/ideas/votes',
            payload_type='vote',
            payload_list=True
        )

    # ----
    # Functions related with comment actions
    # ----

    def comment_comment(self, *args, **kwargs):
        """ :allowed_param: 'commentId', 'text'
        """
        return bind_api(
            api=self,
            path='/comments/{commentId}/comment',
            method='POST',
            payload_type='comment',
            allowed_param=['commentId'],
            post_param=['text']
        )(*args, **kwargs)

    def delete_comment(self, *args, **kwargs):
        """ :allowed_param: 'commentId'
        """
        return bind_api(
            api=self,
            path='/comments/{commentId}/delete',
            method='DELETE',
            payload_type='comment',
            allowed_param=['commentId']
        )(*args, **kwargs)

    @property
    def get_all_comments(self):
        """ :pagination_param: 'page_number', 'page_size'
        """
        return bind_api(
            api=self,
            path='/comments',
            payload_type='comment',
            payload_list=True,
            pagination_param=['page_number', 'page_size']
        )

    def get_votes_comment(self, *args, **kwargs):
        """ :allowed_param: 'commentId'
        """
        return bind_api(
            api=self,
            path='/comments/{commentId}/votes',
            payload_type='vote',
            payload_list=True,
            allowed_param=['commentId']
        )(*args, **kwargs)

    @property
    def get_votes_comments(self):
        return bind_api(
            api=self,
            path='/comments/votes',
            payload_type='vote',
            payload_list=True
        )

    def get_comment(self, *args, **kwargs):
        """ :allowed_param: 'commentId'
        """
        return bind_api(
            api=self,
            path='/comments/{commentId}',
            payload_type='comment',
            allowed_param=['commentId']
        )(*args, **kwargs)

    # ----
    # Functions related with member actions
    # ----

    def create_new_member(self, *args, **kwargs):
        """ :allowed_param: 'name', 'email'
        """
        return bind_api(
            api=self,
            path='/members',
            method='POST',
            payload_type='author',
            post_param=['name', 'email']
        )(*args, **kwargs)

    def get_member_info_by_id(self, *args, **kwargs):
        """ :allowed_param: 'memberId'
        """
        return bind_api(
            api=self,
            path='/members/{memberId}',
            payload_type='author',
            allowed_param=['memberId']
        )(*args, **kwargs)

    def get_member_info_by_name(self, *args, **kwargs):
        """ :allowed_param: 'name'
        """
        return bind_api(
            api=self,
            path='/members/name/{name}',
            payload_type='author',
            payload_list=True,
            allowed_param=['name']
        )(*args, **kwargs)

    def get_member_info_by_email(self, *args, **kwargs):
        """ :allowed_param: 'email'
        """
        return bind_api(
            api=self,
            path='/members/email/{email}',
            payload_type='author',
            allowed_param=['email']
        )(*args, **kwargs)

    def get_member_ideas(self, *args, **kwargs):
        """ :allowed_param: 'memberId'
            :pagination_param: 'page_number', 'page_size'
        """
        return bind_api(
            api=self,
            path='/members/{memberId}/ideas',
            payload_type='idea',
            payload_list=True,
            allowed_param=['memberId'],
            pagination_param=['page_number', 'page_size']
        )(*args, **kwargs)

    def get_votes_ideas_member(self, *args, **kwargs):
        """ :allowed_param: 'memberId'
        """
        return bind_api(
            api=self,
            path='/members/{memberId}/ideas/votes',
            payload_type='vote',
            payload_list=True,
            allowed_param=['memberId']
        )(*args, **kwargs)

    def get_votes_comments_member(self, *args, **kwargs):
        """ :allowed_param: 'memberId'
        """
        return bind_api(
            api=self,
            path='/members/{memberId}/comments/votes',
            payload_type='vote',
            payload_list=True,
            allowed_param=['memberId']
        )(*args, **kwargs)

    def get_comments_member(self, *args, **kwargs):
        """ :allowed_param: 'memberId'
        """
        return bind_api(
            api=self,
            path='/members/{memberId}/comments',
            payload_type='comment',
            payload_list=True,
            allowed_param=['memberId']
        )(*args, **kwargs)


#!/usr/bin/env python

import re
import uuid
from setuptools import setup, find_packages
from pip.req import parse_requirements

VERSIONFILE = "ideascaly/__init__.py"
ver_file = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, ver_file, re.M)

if mo:
    version = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

install_reqs = parse_requirements('requirements.txt', session=uuid.uuid1())
reqs = [str(req.req) for req in install_reqs]

setup(name="ideascaly",
      version=version,
      description="IdeaScale RESTful API Client",
      license="MIT",
      author="Jorge Saldivar",
      author_email="me@jorgesaldivargalli.com",
      url="https://github.com/joausaga/ideascaly",
      packages=find_packages(exclude=['tests', 'env']),
      install_requires=reqs,
      keywords="ideascale api client",
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Libraries',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
      ],
      zip_safe=True)